@@ -83,6 +83,13 @@ class FibreChannelConnector(base.BaseLinuxConnector):
         else:
             wwns = []
 
+        # Convert wwns to lower case
+        wwns = [wwn.lower() for wwn in wwns]
+        if target_wwns:
+            connection_properties['target_wwns'] = wwns
+        elif target_wwn:
+            connection_properties['target_wwn'] = wwns
+
         target_lun = connection_properties.get('target_lun', 0)
         target_luns = connection_properties.get('target_luns')
         if target_luns:
@@ -102,10 +109,10 @@ class FibreChannelConnector(base.BaseLinuxConnector):
             for wwn in wwns:
                 targets.append((wwn, luns[0]))
         else:
-            # Something is wrong, this shouldn't happen
+            # Something is wrong, this shouldn't happen.
             msg = _("Unable to find potential volume paths for FC device "
-                    "with lun: %(lun)s and wwn: %(wwn)s") % {
-                        "lun": target_lun, "wwn": target_wwn}
+                    "with luns: %(luns)s and wwns: %(wwns)s.") % {
+                        "luns": luns, "wwns": wwns}
             LOG.error(msg)
             raise exception.VolumePathsNotFound(msg)
 
@@ -116,8 +123,13 @@ class FibreChannelConnector(base.BaseLinuxConnector):
             wwpn_lun_map[wwpn] = lun
 
         # If there is an initiator_target_map we can update it too
-        if 'initiator_target_map' in connection_properties:
+        if connection_properties.get('initiator_target_map') is not None:
+            # Convert it to lower case
             itmap = connection_properties['initiator_target_map']
+            itmap = {k.lower(): [port.lower() for port in v]
+                     for k, v in itmap.items()}
+            connection_properties['initiator_target_map'] = itmap
+
             new_itmap = dict()
             for init_wwpn in itmap:
                 target_wwpns = itmap[init_wwpn]
@@ -125,7 +137,7 @@ class FibreChannelConnector(base.BaseLinuxConnector):
                 for target_wwpn in target_wwpns:
                     if target_wwpn in wwpn_lun_map:
                         init_targets.append((target_wwpn,
-                                            wwpn_lun_map[target_wwpn]))
+                                             wwpn_lun_map[target_wwpn]))
                 new_itmap[init_wwpn] = init_targets
             connection_properties['initiator_target_lun_map'] = new_itmap
 
@@ -164,7 +176,8 @@ class FibreChannelConnector(base.BaseLinuxConnector):
 
         volume_paths = self.get_volume_paths(connection_properties)
         if volume_paths:
-            return self._linuxscsi.extend_volume(volume_paths)
+            return self._linuxscsi.extend_volume(
+                volume_paths, use_multipath=self.use_multipath)
         else:
             LOG.warning("Couldn't find any volume paths on the host to "
                         "extend volume for %(props)s",
@@ -185,21 +198,19 @@ class FibreChannelConnector(base.BaseLinuxConnector):
         target_wwn - World Wide Name
         target_lun - LUN id of the volume
         """
-        LOG.debug("execute = %s", self._execute)
         device_info = {'type': 'block'}
 
         connection_properties = self._add_targets_to_connection_properties(
             connection_properties)
 
         hbas = self._linuxfc.get_fc_hbas_info()
+        if not hbas:
+            LOG.warning("We are unable to locate any Fibre Channel devices.")
+            raise exception.NoFibreChannelHostsFound()
+
         host_devices = self._get_possible_volume_paths(
             connection_properties, hbas)
 
-        if len(host_devices) == 0:
-            # this is empty because we don't have any FC HBAs
-            LOG.warning("We are unable to locate any Fibre Channel devices")
-            raise exception.NoFibreChannelHostsFound()
-
         # The /dev/disk/by-path/... node is not always present immediately
         # We only need to find the first device.  Once we see the first device
         # multipath will have any others.
@@ -232,10 +243,9 @@ class FibreChannelConnector(base.BaseLinuxConnector):
             _wait_for_device_discovery, host_devices)
         timer.start(interval=2).wait()
 
-        if self.host_device is not None and self.device_name is not None:
-            LOG.debug("Found Fibre Channel volume %(name)s "
-                      "(after %(tries)s rescans)",
-                      {'name': self.device_name, 'tries': self.tries})
+        LOG.debug("Found Fibre Channel volume %(name)s "
+                  "(after %(tries)s rescans.)",
+                  {'name': self.device_name, 'tries': self.tries})
 
         # find out the WWN of the device
         device_wwn = self._linuxscsi.get_scsi_wwn(self.host_device)
@@ -245,9 +255,8 @@ class FibreChannelConnector(base.BaseLinuxConnector):
         # see if the new drive is part of a multipath
         # device.  If so, we'll use the multipath device.
         if self.use_multipath:
-            (device_path, multipath_id) = (super(
-                FibreChannelConnector, self)._discover_mpath_device(
-                device_wwn, connection_properties, self.device_name))
+            (device_path, multipath_id) = self._discover_mpath_device(
+                device_wwn, connection_properties, self.device_name)
             if multipath_id:
                 # only set the multipath_id if we found one
                 device_info['multipath_id'] = multipath_id
@@ -256,7 +265,6 @@ class FibreChannelConnector(base.BaseLinuxConnector):
             device_path = self.host_device
 
         device_info['path'] = device_path
-        LOG.debug("connect_volume returning %s", device_info)
         return device_info
 
     def _get_host_devices(self, possible_devs):
@@ -324,7 +332,8 @@ class FibreChannelConnector(base.BaseLinuxConnector):
         mpath_path = None
         for path in volume_paths:
             real_path = self._linuxscsi.get_name_from_path(path)
-            if self.use_multipath and not mpath_path:
+            if (self.use_multipath and not mpath_path
+                    and self.check_valid_device(path)):
                 wwn = self._linuxscsi.get_scsi_wwn(path)
                 mpath_path = self._linuxscsi.find_multipath_device_path(wwn)
                 if mpath_path:
@@ -333,14 +342,21 @@ class FibreChannelConnector(base.BaseLinuxConnector):
             devices.append(device_info)
 
         LOG.debug("devices to remove = %s", devices)
-        self._remove_devices(connection_properties, devices)
+        self._remove_devices(connection_properties, devices, device_info)
 
-    def _remove_devices(self, connection_properties, devices):
+    def _remove_devices(self, connection_properties, devices, device_info):
         # There may have been more than 1 device mounted
         # by the kernel for this volume.  We have to remove
         # all of them
+        path_used = self._linuxscsi.get_dev_path(connection_properties,
+                                                 device_info)
+        was_multipath = '/pci-' not in path_used
         for device in devices:
-            self._linuxscsi.remove_scsi_device(device["device"])
+            device_path = device['device']
+            flush = self._linuxscsi.requires_flush(device_path,
+                                                   path_used,
+                                                   was_multipath)
+            self._linuxscsi.remove_scsi_device(device_path, flush=flush)
 
     def _get_pci_num(self, hba):
         # NOTE(walter-boring)
