@@ -61,7 +61,7 @@ class LuksEncryptorTestCase(test_cryptsetup.CryptsetupEncryptorTestCase):
 
         mock_execute.assert_has_calls([
             mock.call('cryptsetup', '--batch-mode', 'luksFormat',
-                      '--key-file=-', self.dev_path,
+                      '--type', 'luks1', '--key-file=-', self.dev_path,
                       process_input='passphrase',
                       root_helper=self.root_helper,
                       run_as_root=True, check_exit_code=True, attempts=3),
@@ -124,7 +124,8 @@ class LuksEncryptorTestCase(test_cryptsetup.CryptsetupEncryptorTestCase):
                       root_helper=self.root_helper,
                       run_as_root=True, check_exit_code=True),
             mock.call('cryptsetup', '--batch-mode', 'luksFormat',
-                      '--key-file=-', self.dev_path, process_input=fake_key,
+                      '--type', 'luks1', '--key-file=-', self.dev_path,
+                      process_input=fake_key,
                       root_helper=self.root_helper,
                       run_as_root=True, check_exit_code=True, attempts=3),
             mock.call('cryptsetup', 'luksOpen', '--key-file=-', self.dev_path,
@@ -252,3 +253,62 @@ class LuksEncryptorTestCase(test_cryptsetup.CryptsetupEncryptorTestCase):
                       check_exit_code=True),
         ], any_order=False)
         self.assertEqual(9, mock_execute.call_count)
+
+
+class Luks2EncryptorTestCase(LuksEncryptorTestCase):
+    def _create(self):
+        return luks.Luks2Encryptor(root_helper=self.root_helper,
+                                   connection_info=self.connection_info,
+                                   keymgr=self.keymgr)
+
+    @mock.patch('os_brick.executor.Executor._execute')
+    def test__format_volume(self, mock_execute):
+        self.encryptor._format_volume("passphrase")
+
+        mock_execute.assert_has_calls([
+            mock.call('cryptsetup', '--batch-mode', 'luksFormat',
+                      '--type', 'luks2', '--key-file=-', self.dev_path,
+                      process_input='passphrase',
+                      root_helper=self.root_helper,
+                      run_as_root=True, check_exit_code=True, attempts=3),
+        ])
+
+    @mock.patch('os_brick.executor.Executor._execute')
+    def test_attach_volume_not_formatted(self, mock_execute):
+        fake_key = 'bc37c5eccebe403f9cc2d0dd20dac2bc'
+        self.encryptor._get_key = mock.MagicMock()
+        self.encryptor._get_key.return_value = (
+            test_cryptsetup.fake__get_key(None, fake_key))
+
+        mock_execute.side_effect = [
+            putils.ProcessExecutionError(exit_code=1),  # luksOpen
+            putils.ProcessExecutionError(exit_code=1),  # isLuks
+            mock.DEFAULT,  # luksFormat
+            mock.DEFAULT,  # luksOpen
+            mock.DEFAULT,  # ln
+        ]
+
+        self.encryptor.attach_volume(None)
+
+        mock_execute.assert_has_calls([
+            mock.call('cryptsetup', 'luksOpen', '--key-file=-', self.dev_path,
+                      self.dev_name, process_input=fake_key,
+                      root_helper=self.root_helper,
+                      run_as_root=True, check_exit_code=True),
+            mock.call('cryptsetup', 'isLuks', '--verbose', self.dev_path,
+                      root_helper=self.root_helper,
+                      run_as_root=True, check_exit_code=True),
+            mock.call('cryptsetup', '--batch-mode', 'luksFormat',
+                      '--type', 'luks2', '--key-file=-', self.dev_path,
+                      process_input=fake_key,
+                      root_helper=self.root_helper,
+                      run_as_root=True, check_exit_code=True, attempts=3),
+            mock.call('cryptsetup', 'luksOpen', '--key-file=-', self.dev_path,
+                      self.dev_name, process_input=fake_key,
+                      root_helper=self.root_helper,
+                      run_as_root=True, check_exit_code=True),
+            mock.call('ln', '--symbolic', '--force',
+                      '/dev/mapper/%s' % self.dev_name, self.symlink_path,
+                      root_helper=self.root_helper,
+                      run_as_root=True, check_exit_code=True),
+        ], any_order=False)
