@@ -21,6 +21,7 @@ from oslo_concurrency import processutils as putils
 from os_brick import exception
 from os_brick.initiator.connectors import iscsi
 from os_brick.initiator import linuxscsi
+from os_brick.initiator import utils
 from os_brick.privileged import rootwrap as priv_rootwrap
 from os_brick.tests.initiator import test_connector
 
@@ -70,12 +71,17 @@ class ISCSIConnectorTestCase(test_connector.ConnectorTestCase):
                     ('tcp:', 'session3', 'ip3:port3', '1', 'tgt3')]
         self.assertListEqual(expected, res)
 
-    @mock.patch.object(iscsi.ISCSIConnector, '_run_iscsi_session',
-                       return_value=(None, 'error'))
-    def test_get_iscsi_sessions_full_error(self, sessions_mock):
+    @mock.patch.object(iscsi.ISCSIConnector, '_run_iscsi_session')
+    def test_get_iscsi_sessions_full_stderr(self, sessions_mock):
+        iscsiadm_result = ('tcp: [session1] ip1:port1,1 tgt1 (non-flash)\n'
+                           'tcp: [session2] ip2:port2,-1 tgt2 (non-flash)\n'
+                           'tcp: [session3] ip3:port3,1 tgt3\n')
+        sessions_mock.return_value = (iscsiadm_result, 'error')
         res = self.connector._get_iscsi_sessions_full()
-        self.assertEqual([], res)
-        sessions_mock.assert_called()
+        expected = [('tcp:', 'session1', 'ip1:port1', '1', 'tgt1'),
+                    ('tcp:', 'session2', 'ip2:port2', '-1', 'tgt2'),
+                    ('tcp:', 'session3', 'ip3:port3', '1', 'tgt3')]
+        self.assertListEqual(expected, res)
 
     @mock.patch.object(iscsi.ISCSIConnector, '_get_iscsi_sessions_full')
     def test_get_iscsi_sessions(self, sessions_mock):
@@ -538,7 +544,8 @@ class ISCSIConnectorTestCase(test_connector.ConnectorTestCase):
         cleanup_mock.assert_called_once_with(
             mock.sentinel.con_props,
             force=mock.sentinel.Force,
-            ignore_errors=mock.sentinel.ignore_errors)
+            ignore_errors=mock.sentinel.ignore_errors,
+            device_info=mock.sentinel.dev_info)
 
     @ddt.data(True, False)
     @mock.patch.object(iscsi.ISCSIConnector, '_get_transport')
@@ -649,13 +656,21 @@ class ISCSIConnectorTestCase(test_connector.ConnectorTestCase):
             ['-m', 'discoverydb', '-o', 'show', '-P', 1])
         transport_mock.assert_called_once_with()
 
+    @ddt.data(('/dev/sda', False),
+              ('/dev/disk/by-id/scsi-WWID', False),
+              ('/dev/dm-11', True),
+              ('/dev/disk/by-id/dm-uuid-mpath-MPATH', True))
+    @ddt.unpack
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_dev_path')
     @mock.patch.object(iscsi.ISCSIConnector, '_disconnect_connection')
     @mock.patch.object(iscsi.ISCSIConnector, '_get_connection_devices')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'flush_multipath_device')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'remove_connection',
                        return_value=None)
-    def test_cleanup_connection(self, remove_mock, flush_mock, con_devs_mock,
-                                discon_mock):
+    def test_cleanup_connection(self, path_used, was_multipath, remove_mock,
+                                flush_mock, con_devs_mock, discon_mock,
+                                get_dev_path_mock):
+        get_dev_path_mock.return_value = path_used
         # Return an ordered dicts instead of normal dict for discon_mock.assert
         con_devs_mock.return_value = collections.OrderedDict((
             (('ip1:port1', 'tgt1'), ({'sda'}, set())),
@@ -666,12 +681,16 @@ class ISCSIConnectorTestCase(test_connector.ConnectorTestCase):
                                'use_multipath') as use_mp_mock:
             self.connector._cleanup_connection(
                 self.CON_PROPS, ips_iqns_luns=mock.sentinel.ips_iqns_luns,
-                force=False, ignore_errors=False)
+                force=False, ignore_errors=False,
+                device_info=mock.sentinel.device_info)
 
+        get_dev_path_mock.called_once_with(self.CON_PROPS,
+                                           mock.sentinel.device_info)
         con_devs_mock.assert_called_once_with(self.CON_PROPS,
                                               mock.sentinel.ips_iqns_luns)
         remove_mock.assert_called_once_with({'sda', 'sdb'}, use_mp_mock,
-                                            False, mock.ANY)
+                                            False, mock.ANY,
+                                            path_used, was_multipath)
         discon_mock.assert_called_once_with(
             self.CON_PROPS,
             [('ip1:port1', 'tgt1'), ('ip3:port3', 'tgt3')],
@@ -707,7 +726,8 @@ class ISCSIConnectorTestCase(test_connector.ConnectorTestCase):
         con_devs_mock.assert_called_once_with(self.CON_PROPS,
                                               mock.sentinel.ips_iqns_luns)
         remove_mock.assert_called_once_with({'sda', 'sdb'}, use_mp_mock,
-                                            mock.sentinel.force, mock.ANY)
+                                            mock.sentinel.force, mock.ANY,
+                                            '', False)
         discon_mock.assert_called_once_with(
             self.CON_PROPS,
             [('ip1:port1', 'tgt1'), ('ip3:port3', 'tgt3')],
@@ -968,6 +988,7 @@ Setting up iSCSI targets: unused
             [('tcp:', 'session1', 'ip1:port1', '1', 'tgt'),
              ('tcp:', session, 'ip1:port1', '-1', 'tgt1')]
         ]
+        utils.ISCSI_SUPPORTS_MANUAL_SCAN = None
         with mock.patch.object(self.connector, '_execute') as exec_mock:
             exec_mock.side_effect = [('', 'error'), ('', None),
                                      ('', None), ('', None),
@@ -977,6 +998,40 @@ Setting up iSCSI targets: unused
         # True refers to "manual scans", since the call to update
         # node.session.scan didn't fail they are set to manual
         self.assertEqual((session, True), res)
+        self.assertTrue(utils.ISCSI_SUPPORTS_MANUAL_SCAN)
+        prefix = 'iscsiadm -m node -T tgt1 -p ip1:port1'
+        expected_cmds = [
+            prefix,
+            prefix + ' --interface default --op new',
+            prefix + ' --op update -n node.session.scan -v manual',
+            prefix + ' --login',
+            prefix + ' --op update -n node.startup -v automatic'
+        ]
+        actual_cmds = [' '.join(args[0]) for args in exec_mock.call_args_list]
+        self.assertListEqual(expected_cmds, actual_cmds)
+        self.assertEqual(2, get_sessions_mock.call_count)
+
+    @mock.patch.object(iscsi.ISCSIConnector, '_get_iscsi_sessions_full')
+    def test_connect_to_iscsi_portal_ip_case_insensitive(self,
+                                                         get_sessions_mock):
+        """Connect creating node and session."""
+        session = 'session2'
+        get_sessions_mock.side_effect = [
+            [('tcp:', 'session1', 'iP1:port1', '1', 'tgt')],
+            [('tcp:', 'session1', 'Ip1:port1', '1', 'tgt'),
+             ('tcp:', session, 'IP1:port1', '-1', 'tgt1')]
+        ]
+        utils.ISCSI_SUPPORTS_MANUAL_SCAN = None
+        with mock.patch.object(self.connector, '_execute') as exec_mock:
+            exec_mock.side_effect = [('', 'error'), ('', None),
+                                     ('', None), ('', None),
+                                     ('', None)]
+            res = self.connector._connect_to_iscsi_portal(self.CON_PROPS)
+
+        # True refers to "manual scans", since the call to update
+        # node.session.scan didn't fail they are set to manual
+        self.assertEqual((session, True), res)
+        self.assertTrue(utils.ISCSI_SUPPORTS_MANUAL_SCAN)
         prefix = 'iscsiadm -m node -T tgt1 -p ip1:port1'
         expected_cmds = [
             prefix,
@@ -998,9 +1053,11 @@ Setting up iSCSI targets: unused
         con_props = self.CON_PROPS.copy()
         con_props.update(auth_method='CHAP', auth_username='user',
                          auth_password='pwd')
+        utils.ISCSI_SUPPORTS_MANUAL_SCAN = None
         res = self.connector._connect_to_iscsi_portal(con_props)
         # False refers to "manual scans", so we have manual iscsi scans
         self.assertEqual((session, True), res)
+        self.assertTrue(utils.ISCSI_SUPPORTS_MANUAL_SCAN)
         prefix = 'iscsiadm -m node -T tgt1 -p ip1:port1'
         expected_cmds = [
             prefix,
@@ -1054,6 +1111,27 @@ Setting up iSCSI targets: unused
             (('ip1:port1', 'tgt1', 4),),
             force=True, ignore_errors=True)
 
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_sysfs_wwn', return_value='')
+    @mock.patch.object(iscsi.ISCSIConnector, '_connect_vol')
+    @mock.patch.object(iscsi.ISCSIConnector, '_cleanup_connection')
+    @mock.patch('time.sleep')
+    def test_connect_single_volume_no_wwn(self, sleep_mock, cleanup_mock,
+                                          connect_mock, get_wwn_mock):
+        def my_connect(rescans, props, data):
+            data['found_devices'].append('sdz')
+
+        connect_mock.side_effect = my_connect
+
+        res = self.connector._connect_single_volume(self.CON_PROPS)
+
+        expected = {'type': 'block', 'scsi_wwn': '', 'path': '/dev/sdz'}
+        self.assertEqual(expected, res)
+        get_wwn_mock.assert_has_calls([mock.call(['sdz'])] * 10)
+        self.assertEqual(10, get_wwn_mock.call_count)
+        sleep_mock.assert_has_calls([mock.call(1)] * 10)
+        self.assertEqual(10, sleep_mock.call_count)
+        cleanup_mock.assert_not_called()
+
     @staticmethod
     def _get_connect_vol_data():
         return {'stop_connecting': False, 'num_logins': 0, 'failed_logins': 0,
@@ -1277,12 +1355,13 @@ Setting up iSCSI targets: unused
         find_dm_mock.assert_not_called()
         self.assertEqual(12, connect_mock.call_count)
 
-    @mock.patch('time.sleep', mock.Mock())
+    @mock.patch('time.sleep')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'scan_iscsi')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'device_name_by_hctl',
                        return_value='sda')
     @mock.patch.object(iscsi.ISCSIConnector, '_connect_to_iscsi_portal')
-    def test_connect_vol(self, connect_mock, dev_name_mock, scan_mock):
+    def test_connect_vol(self, connect_mock, dev_name_mock, scan_mock,
+                         sleep_mock):
         lscsi = self.connector._linuxscsi
         data = self._get_connect_vol_data()
         hctl = [mock.sentinel.host, mock.sentinel.channel,
@@ -1305,8 +1384,72 @@ Setting up iSCSI targets: unused
                                     mock.call(mock.sentinel.session,
                                               self.CON_PROPS['target_lun'])])
 
-        scan_mock.assert_called_once_with(*hctl)
+        scan_mock.assert_not_called()
         dev_name_mock.assert_called_once_with(mock.sentinel.session, hctl)
+        sleep_mock.assert_called_once_with(1)
+
+    @mock.patch('time.sleep')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'scan_iscsi')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'device_name_by_hctl',
+                       side_effect=(None, None, None, None, 'sda'))
+    @mock.patch.object(iscsi.ISCSIConnector, '_connect_to_iscsi_portal')
+    def test_connect_vol_rescan(self, connect_mock, dev_name_mock, scan_mock,
+                                sleep_mock):
+        lscsi = self.connector._linuxscsi
+        data = self._get_connect_vol_data()
+        hctl = [mock.sentinel.host, mock.sentinel.channel,
+                mock.sentinel.target, mock.sentinel.lun]
+
+        connect_mock.return_value = (mock.sentinel.session, False)
+
+        with mock.patch.object(lscsi, 'get_hctl',
+                               return_value=hctl) as hctl_mock:
+            self.connector._connect_vol(3, self.CON_PROPS, data)
+
+        expected = self._get_connect_vol_data()
+        expected.update(num_logins=1, stopped_threads=1,
+                        found_devices=['sda'], just_added_devices=['sda'])
+        self.assertDictEqual(expected, data)
+
+        connect_mock.assert_called_once_with(self.CON_PROPS)
+        hctl_mock.assert_called_once_with(mock.sentinel.session,
+                                          self.CON_PROPS['target_lun'])
+
+        scan_mock.assert_called_once_with(*hctl)
+        self.assertEqual(5, dev_name_mock.call_count)
+        self.assertEqual(4, sleep_mock.call_count)
+
+    @mock.patch('time.sleep')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'scan_iscsi')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'device_name_by_hctl',
+                       side_effect=(None, None, None, None, 'sda'))
+    @mock.patch.object(iscsi.ISCSIConnector, '_connect_to_iscsi_portal')
+    def test_connect_vol_manual(self, connect_mock, dev_name_mock, scan_mock,
+                                sleep_mock):
+        lscsi = self.connector._linuxscsi
+        data = self._get_connect_vol_data()
+        hctl = [mock.sentinel.host, mock.sentinel.channel,
+                mock.sentinel.target, mock.sentinel.lun]
+
+        # Simulate manual scan
+        connect_mock.return_value = (mock.sentinel.session, True)
+
+        with mock.patch.object(lscsi, 'get_hctl',
+                               return_value=hctl) as hctl_mock:
+            self.connector._connect_vol(3, self.CON_PROPS, data)
+
+        expected = self._get_connect_vol_data()
+        expected.update(num_logins=1, stopped_threads=1,
+                        found_devices=['sda'], just_added_devices=['sda'])
+        self.assertDictEqual(expected, data)
+
+        connect_mock.assert_called_once_with(self.CON_PROPS)
+        hctl_mock.assert_called_once_with(mock.sentinel.session,
+                                          self.CON_PROPS['target_lun'])
+
+        self.assertEqual(2, scan_mock.call_count)
+        self.assertEqual(5, dev_name_mock.call_count)
+        self.assertEqual(4, sleep_mock.call_count)
 
     @mock.patch.object(iscsi.ISCSIConnector, '_connect_to_iscsi_portal',
                        return_value=(None, False))
@@ -1496,6 +1639,20 @@ Setting up iSCSI targets: unused
                                   iqn3: node_startup3}
         self.assertEqual(node_startups, expected_node_startups)
 
+    @mock.patch.object(iscsi.ISCSIConnector, '_execute')
+    def test_get_node_startup_values_no_nodes(self, exec_mock):
+        connection_properties = {'target_portal': 'ip1:port1'}
+        no_nodes_output = ''
+        no_nodes_err = 'iscsiadm: No records found\n'
+        exec_mock.return_value = (no_nodes_output, no_nodes_err)
+        res = self.connector._get_node_startup_values(connection_properties)
+        self.assertEqual({}, res)
+        exec_mock.assert_called_once_with(
+            'iscsiadm', '-m', 'node', '--op', 'show', '-p',
+            connection_properties['target_portal'],
+            root_helper=self.connector._root_helper, run_as_root=True,
+            check_exit_code=(0, 21))
+
     @mock.patch.object(iscsi.ISCSIConnector, '_get_node_startup_values')
     @mock.patch.object(iscsi.ISCSIConnector, '_iscsiadm_update')
     def test_recover_node_startup_values(self, iscsiadm_update_mock,
@@ -1508,8 +1665,8 @@ Setting up iSCSI targets: unused
         iqn1 = 'iqn.2010-10.org.openstack:%s' % name1
         iqn2 = 'iqn.2010-10.org.openstack:%s' % name2
         iqn3 = 'iqn.2010-10.org.openstack:%s' % name3
-        connection_properties = self.iscsi_connection(vol, [location], [iqn1])
-        recover_connection = self.iscsi_connection(vol, [location], [iqn2])
+        connection_properties = self.iscsi_connection(vol, [location], iqn1)
+        recover_connection = self.iscsi_connection(vol, [location], iqn2)
 
         node_startup1 = "manual"
         node_startup2 = "automatic"
