@@ -50,9 +50,17 @@ class LinuxFCTestCase(base.TestCase):
         connection_properties = {
             'initiator_target_map': {'50014380186af83c': ['514f0c50023f6c00'],
                                      '50014380186af83e': ['514f0c50023f6c01']},
+            'initiator_target_lun_map': {
+                '50014380186af83c': [('514f0c50023f6c00', 1)],
+                '50014380186af83e': [('514f0c50023f6c01', 1)]
+            },
             'target_discovered': False,
             'target_lun': 1,
-            'target_wwn': ['514f0c50023f6c00', '514f0c50023f6c01']
+            'target_wwn': ['514f0c50023f6c00', '514f0c50023f6c01'],
+            'targets': [
+                ('514f0c50023f6c00', 1),
+                ('514f0c50023f6c01', 1),
+            ]
         }
 
         hbas = [
@@ -69,6 +77,7 @@ class LinuxFCTestCase(base.TestCase):
         ]
         if not zone_manager:
             del connection_properties['initiator_target_map']
+            del connection_properties['initiator_target_lun_map']
         return hbas, connection_properties
 
     def test__get_hba_channel_scsi_target_single_wwpn(self):
@@ -76,6 +85,7 @@ class LinuxFCTestCase(base.TestCase):
                            '')
         hbas, con_props = self.__get_rescan_info()
         con_props['target_wwn'] = con_props['target_wwn'][0]
+        con_props['targets'] = con_props['targets'][0:1]
         with mock.patch.object(self.lfc, '_execute',
                                return_value=execute_results) as execute_mock:
             res = self.lfc._get_hba_channel_scsi_target(hbas[0], con_props)
@@ -83,22 +93,91 @@ class LinuxFCTestCase(base.TestCase):
                 'grep -Gil "514f0c50023f6c00" '
                 '/sys/class/fc_transport/target6:*/port_name',
                 shell=True)
-        expected = [['0', '1']]
+        expected = [['0', '1', 1]]
         self.assertListEqual(expected, res)
 
-    def test__get_hba_channel_scsi_target_multiple_wwpn(self):
-        execute_results = ('/sys/class/fc_transport/target6:0:1/port_name\n'
-                           '/sys/class/fc_transport/target6:0:2/port_name\n',
+    def test__get_hba_channel_scsi_target_with_initiator_target_map(self):
+        execute_results = ('/sys/class/fc_transport/target6:0:1/port_name\n',
                            '')
-        hbas, con_props = self.__get_rescan_info()
+        hbas, con_props = self.__get_rescan_info(zone_manager=True)
+        con_props['target_wwn'] = con_props['target_wwn'][0]
+        con_props['targets'] = con_props['targets'][0:1]
+        hbas[0]['port_name'] = '50014380186af83e'
         with mock.patch.object(self.lfc, '_execute',
                                return_value=execute_results) as execute_mock:
             res = self.lfc._get_hba_channel_scsi_target(hbas[0], con_props)
             execute_mock.assert_called_once_with(
-                'grep -Gil "514f0c50023f6c00\|514f0c50023f6c01" '
+                'grep -Gil "514f0c50023f6c01" '
                 '/sys/class/fc_transport/target6:*/port_name',
                 shell=True)
-        expected = [['0', '1'], ['0', '2']]
+        expected = [['0', '1', 1]]
+        self.assertListEqual(expected, res)
+
+    def test__get_hba_channel_scsi_target_with_initiator_target_map_none(self):
+        execute_results = ('/sys/class/fc_transport/target6:0:1/port_name\n',
+                           '')
+        hbas, con_props = self.__get_rescan_info()
+        con_props['target_wwn'] = con_props['target_wwn'][0]
+        con_props['targets'] = con_props['targets'][0:1]
+        con_props['initiator_target_map'] = None
+        hbas[0]['port_name'] = '50014380186af83e'
+        with mock.patch.object(self.lfc, '_execute',
+                               return_value=execute_results) as execute_mock:
+            res = self.lfc._get_hba_channel_scsi_target(hbas[0], con_props)
+            execute_mock.assert_called_once_with(
+                'grep -Gil "514f0c50023f6c00" '
+                '/sys/class/fc_transport/target6:*/port_name',
+                shell=True)
+        expected = [['0', '1', 1]]
+        self.assertListEqual(expected, res)
+
+    def test__get_hba_channel_scsi_target_multiple_wwpn(self):
+        execute_results = [
+            ['/sys/class/fc_transport/target6:0:1/port_name\n', ''],
+            ['/sys/class/fc_transport/target6:0:2/port_name\n', ''],
+        ]
+        hbas, con_props = self.__get_rescan_info()
+        with mock.patch.object(self.lfc, '_execute',
+                               side_effect=execute_results) as execute_mock:
+            res = self.lfc._get_hba_channel_scsi_target(hbas[0], con_props)
+            expected_cmds = [
+                mock.call('grep -Gil "514f0c50023f6c00" '
+                          '/sys/class/fc_transport/target6:*/port_name',
+                          shell=True),
+                mock.call('grep -Gil "514f0c50023f6c01" '
+                          '/sys/class/fc_transport/target6:*/port_name',
+                          shell=True),
+            ]
+            execute_mock.assert_has_calls(expected_cmds)
+
+        expected = [['0', '1', 1], ['0', '2', 1]]
+        self.assertListEqual(expected, res)
+
+    def test__get_hba_channel_scsi_target_multiple_wwpn_and_luns(self):
+        execute_results = [
+            ['/sys/class/fc_transport/target6:0:1/port_name\n', ''],
+            ['/sys/class/fc_transport/target6:0:2/port_name\n', ''],
+        ]
+        hbas, con_props = self.__get_rescan_info()
+        con_props['target_lun'] = [1, 7]
+        con_props['targets'] = [
+            ('514f0c50023f6c00', 1),
+            ('514f0c50023f6c01', 7),
+        ]
+        with mock.patch.object(self.lfc, '_execute',
+                               side_effect=execute_results) as execute_mock:
+            res = self.lfc._get_hba_channel_scsi_target(hbas[0], con_props)
+            expected_cmds = [
+                mock.call('grep -Gil "514f0c50023f6c00" '
+                          '/sys/class/fc_transport/target6:*/port_name',
+                          shell=True),
+                mock.call('grep -Gil "514f0c50023f6c01" '
+                          '/sys/class/fc_transport/target6:*/port_name',
+                          shell=True),
+            ]
+            execute_mock.assert_has_calls(expected_cmds)
+
+        expected = [['0', '1', 1], ['0', '2', 7]]
         self.assertListEqual(expected, res)
 
     def test__get_hba_channel_scsi_target_zone_manager(self):
@@ -112,7 +191,7 @@ class LinuxFCTestCase(base.TestCase):
                 'grep -Gil "514f0c50023f6c00" '
                 '/sys/class/fc_transport/target6:*/port_name',
                 shell=True)
-        expected = [['0', '1']]
+        expected = [['0', '1', 1]]
         self.assertListEqual(expected, res)
 
     def test__get_hba_channel_scsi_target_not_found(self):
@@ -135,12 +214,22 @@ class LinuxFCTestCase(base.TestCase):
                 'grep -Gil "514f0c50023f6c00" '
                 '/sys/class/fc_transport/target6:*/port_name',
                 shell=True)
-        self.assertIsNone(res)
+        self.assertEqual(res, [['-', '-', 1]])
 
-    def test_rescan_hosts(self):
-        get_chan_results = [[['2', '3'], ['4', '5']], [['6', '7']]]
+    def test_rescan_hosts_initiator_map(self):
+        """Test FC rescan with initiator map and not every HBA connected."""
+        get_chan_results = [[['2', '3', 1], ['4', '5', 1]], [['6', '7', 1]]]
 
         hbas, con_props = self.__get_rescan_info(zone_manager=True)
+
+        # This HBA is not in the initiator map, so we should not scan it or try
+        # to get the channel and target
+        hbas.append({'device_path': ('/sys/devices/pci0000:00/0000:00:02.0/'
+                                     '0000:04:00.2/host8/fc_host/host8'),
+                     'host_device': 'host8',
+                     'node_name': '50014380186af83g',
+                     'port_name': '50014380186af83h'})
+
         with mock.patch.object(self.lfc, '_execute',
                                return_value=None) as execute_mock, \
             mock.patch.object(self.lfc, '_get_hba_channel_scsi_target',
@@ -165,10 +254,83 @@ class LinuxFCTestCase(base.TestCase):
                               mock.call(hbas[1], con_props)]
             mock_get_chan.assert_has_calls(expected_calls)
 
-    def test_rescan_hosts_wildcard(self):
+    def test_rescan_hosts_single_wwnn(self):
+        """Test FC rescan with no initiator map and single WWNN for ports."""
+        get_chan_results = [
+            [['2', '3', 1], ['4', '5', 1]],
+            [['6', '7', 1]],
+            [['-', '-', 1]],
+        ]
+
+        hbas, con_props = self.__get_rescan_info(zone_manager=False)
+
+        # This HBA is the one that is not included in the single WWNN.
+        hbas.append({'device_path': ('/sys/devices/pci0000:00/0000:00:02.0/'
+                                     '0000:04:00.2/host8/fc_host/host8'),
+                     'host_device': 'host8',
+                     'node_name': '50014380186af83g',
+                     'port_name': '50014380186af83h'})
+
+        with mock.patch.object(self.lfc, '_execute',
+                               return_value=None) as execute_mock, \
+            mock.patch.object(self.lfc, '_get_hba_channel_scsi_target',
+                              side_effect=get_chan_results) as mock_get_chan:
+
+            self.lfc.rescan_hosts(hbas, con_props)
+            expected_commands = [
+                mock.call('tee', '-a', '/sys/class/scsi_host/host6/scan',
+                          process_input='2 3 1',
+                          root_helper=None, run_as_root=True),
+                mock.call('tee', '-a', '/sys/class/scsi_host/host6/scan',
+                          process_input='4 5 1',
+                          root_helper=None, run_as_root=True),
+                mock.call('tee', '-a', '/sys/class/scsi_host/host7/scan',
+                          process_input='6 7 1',
+                          root_helper=None, run_as_root=True)]
+
+            execute_mock.assert_has_calls(expected_commands)
+            self.assertEqual(len(expected_commands), execute_mock.call_count)
+
+            expected_calls = [mock.call(hbas[0], con_props),
+                              mock.call(hbas[1], con_props)]
+            mock_get_chan.assert_has_calls(expected_calls)
+
+    def test_rescan_hosts_initiator_map_single_wwnn(self):
+        """Test FC rescan with initiator map and single WWNN."""
+        get_chan_results = [[['2', '3', 1], ['4', '5', 1]], [['-', '-', 1]]]
+
         hbas, con_props = self.__get_rescan_info(zone_manager=True)
+
+        with mock.patch.object(self.lfc, '_execute',
+                               return_value=None) as execute_mock, \
+            mock.patch.object(self.lfc, '_get_hba_channel_scsi_target',
+                              side_effect=get_chan_results) as mock_get_chan:
+
+            self.lfc.rescan_hosts(hbas, con_props)
+            expected_commands = [
+                mock.call('tee', '-a', '/sys/class/scsi_host/host6/scan',
+                          process_input='2 3 1',
+                          root_helper=None, run_as_root=True),
+                mock.call('tee', '-a', '/sys/class/scsi_host/host6/scan',
+                          process_input='4 5 1',
+                          root_helper=None, run_as_root=True)]
+
+            execute_mock.assert_has_calls(expected_commands)
+            self.assertEqual(len(expected_commands), execute_mock.call_count)
+
+            expected_calls = [mock.call(hbas[0], con_props),
+                              mock.call(hbas[1], con_props)]
+            mock_get_chan.assert_has_calls(expected_calls)
+
+    def test_rescan_hosts_wildcard(self):
+        """Test when we don't have initiator map or target is single WWNN."""
+        get_chan_results = [[['-', '-', 1]], [['-', '-', 1]]]
+        hbas, con_props = self.__get_rescan_info(zone_manager=True)
+        # Remove the initiator map
+        con_props.pop('initiator_target_map')
+        con_props.pop('initiator_target_lun_map')
         with mock.patch.object(self.lfc, '_get_hba_channel_scsi_target',
-                               side_effect=(None, [])), \
+                               side_effect=get_chan_results), \
             mock.patch.object(self.lfc, '_execute',
                               side_effect=None) as execute_mock:
 
